package cnf;

import java.util.HashMap;
import java.util.Iterator;
import cnf.Bool;

public class Environment {

	protected HashMap<Integer, Bool> values;
	
	/**
	 * Create a new, empty environment
	 */
	public Environment()
	{
		values = new HashMap<Integer, Bool>();
	}
	
	/**
	 * Create a copy of the provided environment
	 */
	public Environment(Environment copy)
	{
		values = new HashMap<Integer, Bool>();
		values.putAll(copy.values);	
	}
	
	/**
	 * Picks an unassigned variable
	 * @return any unassigned variable or -1 if none could be found
	 */
	public int choose()
	{
		int choice = -1;
		
		Iterator<Integer> ids = values.keySet().iterator();
		
		while(choice==-1 && ids.hasNext())
		{
			Integer key = ids.next();
			if(values.get(key.intValue())==Bool.UNDEFINED)
				choice = key.intValue();
		}
		
		return(choice);
	}
	
	/**
	 * Initializes numVars variables
	 * @param start
	 * @param end
	 */
	public void initializeRange(int numVars)
	{
		for(int i=0; i<numVars; i++)
		{
			this.add(i, Bool.UNDEFINED);
		}
	}
	
	/**
	 * Adds a variable to the environment
	 * @param var - Literal id to be added
	 * @param value - value (can be TRUE, FALSE or UNDEFINED)
	 */
	public void add(int var, Bool value)
	{
		Integer ID = new Integer(var);
		values.put(ID, value);
	}
	
	/**
	 * Returns the binding for a given value, UNDEFINED if no
	 * binding yet exists
	 * @param id
	 * @return
	 */
	public Bool eval(int id)
	{
		Bool val = values.get(new Integer(id));
		if(val==null)
			val =  Bool.UNDEFINED;
		return(val);
	}
	
	public void display()
	{
		Iterator<Integer> ids = values.keySet().iterator();
		
		while(ids.hasNext())
		{
			Integer key = ids.next();
			System.err.printf("%d: %s\n", key.intValue(), values.get(key.intValue()).toString());
		}
	}
}
