package friendly;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Map;
import java.util.LinkedHashMap;

/**
 * The friendly ear program chats with the user until the user types "bye", then
 * terminates.
 */
public class Main {

	public static void main(String[] args) {
		
		//first, initialize the friendly server connection.
		ServerSocket serverSocket = null;
		boolean serverON = true;
		try {
			serverSocket = new ServerSocket(4444);

			
			//now wait for a connection
			while(serverON)
			{
				Socket socket = null;
				try {
					System.out.println("Accepting connections...");
					socket = serverSocket.accept();
				} catch (IOException e) {
					System.err.println("Accept failed.");
					System.exit(1);
				}		
		
				//Create a reader and writer
				BufferedReader reader = null;
				PrintWriter writer = null;
				
				//Make some canned responses
				Map<String, String> cannedResponses = new LinkedHashMap<String, String>();
				cannedResponses.put("I think it's going to rain today.", "No way! Let's go to the beach!");
				cannedResponses.put("Hey, what's happening?", "We need to talk about your TPS reports.");
				cannedResponses.put("I think I'm going to lose it.", "Uh-oh, looks like someone has a case of the Mondays!");
				cannedResponses.put("Ring ring ring...", "Bananaphone!");
		
				try {
					
					// Create reader and writer, connect them to socket.
					reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
					writer = new PrintWriter(socket.getOutputStream(), true);
					
					// Acknowledge connection.
					writer.append("Connected to server. Type a line of input followed by the ENTER key.");
					writer.append(System.getProperty("line.separator"));
					writer.println("Hello friend! How are you today?");
					writer.flush();			
					
					// Read the first line.
					String line = reader.readLine();
		
					while (line != null) { // line==null means the end of the stream
											// was reached.
		
						// If user typed "bye", exit the loop.
						// Use `toLowerCase' and `startsWith' to allow variations like
						// `BYE',`Bye', `bye!', etc.
						if (line.toLowerCase().startsWith("bye")) {
							writer.println("Well, bye then.");
							writer.flush();
							break;
						}
						
						if (line.toLowerCase().startsWith("kill server"))
						{
							writer.println("Daisy, daiiisy, giive meeee yourrr ansswerr... zzzz");
							writer.flush();
							serverON = false;
							break;
						}
		
						if (cannedResponses.get(line) != null)
							writer.println(cannedResponses.get(line));
						else if (line.endsWith("?")) {
							// User asked a question.
							writer.println("Interesting question.");
						} else {
							writer.println("I see. Why is that?");
						}
						writer.flush();
		
						// Read next line.
						line = reader.readLine();
					}
				
					//Terminate the connection
					writer.println("Closing connection... ");
					writer.flush();
					reader.close();
					writer.close();
					socket.close();
		
				} catch (IOException e) {
					System.out.println("I/O error: " + e.getMessage());
					System.exit(1);
				}
			}
	
			serverSocket.close();	
			System.out.println("Terminating connection...");
		} catch (IOException e) {
			System.err.println("Could not listen on port: 4444.");
			System.exit(1);
		}
	}
}
