package friendly.multi;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;
import java.util.LinkedHashMap;

/**
 * Handles one user's Friendly session.
 */
public class FriendlyClientHandler implements Runnable {
    private BufferedReader reader;
    private PrintWriter writer;
    private Map<String, String> cannedResponses;
    
    public FriendlyClientHandler(BufferedReader reader, PrintWriter writer) {
        this.reader = reader;
        this.writer = writer;
        //Make some canned responses
		cannedResponses = new LinkedHashMap<String, String>();
		cannedResponses.put("I think it's going to rain today.", "No way! Let's go to the beach!");
		cannedResponses.put("Hey, what's happening?", "We need to talk about your TPS reports.");
		cannedResponses.put("I think I'm going to lose it.", "Uh-oh, looks like someone has a case of the Mondays!");
		cannedResponses.put("Ring ring ring...", "Bananaphone!");
    }
    
	public void run() {
        System.out.println("Connection opened");
        try {
            // Read the first line.
            String line = reader.readLine();
            
            writer.append("Connected to server. Type a line of input followed by the ENTER key.");
			writer.append(System.getProperty("line.separator"));
			writer.println("Hello friend! How are you today?");
			writer.flush();	
    
            while (line != null) { // line==null means the end of the stream
                                    // was reached.
            	if (line.toLowerCase().startsWith("bye")) {
					writer.println("Well, bye then.");
					writer.flush();
					break;
				}
				
				if (line.toLowerCase().startsWith("kill server"))
				{
					writer.println("Daisy, daiiisy, giive meeee yourrr ansswerr... zzzz");
					writer.flush();
					break;
				}

				if (cannedResponses.get(line) != null)
					writer.println(cannedResponses.get(line));
				else if (line.endsWith("?")) {
					// User asked a question.
					writer.println("Interesting question.");
				} else {
					writer.println("I see. Why is that?");
				}
				writer.flush();
				
	            line = reader.readLine();
            }
            reader.close();
            writer.close();
		} catch (IOException e) {
			
			e.printStackTrace();
		}
		System.out.println("Connection closed");
	}
}