package multipart;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.lang.String;
import java.net.URL;
import multipart.Manifest;

/**
 * 
 * This implements manifest, looking for files which consist of
 * URLs of mirrors separated by --
 * @author scottyoung
 *
 */
public class ManifestMachine implements Manifest {

	final static String MANIFEST_FINISHED = "MANIFEST_FINISHED";
	BufferedReader contents;
	boolean isFinished = false;
	
	/**
	 * Create a new manifest, given a URL
	 */
	public ManifestMachine(String url)
	{
		try{
			contents = new BufferedReader(new InputStreamReader(new URL(url).openStream()));
		}catch(IOException e){
			System.err.println("Unable to create manifest from URL, "+e.getMessage());
		}
	}
	/**
	 * Returns the URL of the next file (manifest OR data) in 
	 * the stream. Throws an error if the manifest file is bad
	 * or if none of the mirrors for a component are available
	 * Returns MANIFEST_FINISHED flag upon successfully reaching
	 * end-of-file
	 */
	public String get() throws RuntimeException
	{
		String nextLine = null, line = null;
		URL url;
		InputStream tester;
		//invalid controls walkthrough of mirrors, forceReturn forces an error if all mirrors fail
		boolean invalid = true, forceReturn = false;
		
		if(isFinished)
			return(MANIFEST_FINISHED);
		
		try{
			line = contents.readLine();
		}catch(IOException e){
			System.err.println("ManifestMachine get() failed to read first line, "+e.getMessage());
		}
		
		//continue reading lines until you get a valid URL
		while(invalid)
		{
			try{
				if(line.equals("--") && forceReturn){
					throw(new RuntimeException("Hit next file segment before finding valid mirror"));
				}
				if(line!=null)
				{
					url = new URL(line);
					tester = url.openStream();
				}
				invalid = false;
				break;
			}catch(IOException e){
				invalid = true;
				forceReturn = true; //since we've encountered a broken link, we MUST find a file or return an error
				try{
					line = contents.readLine();
				}catch(IOException f){
					System.err.println("ManifestMachine get() failed to read mirror line, "+f.getMessage());
				}
			}
		}
		
		if(line==null)
		{
			if(forceReturn)
			{
				throw(new RuntimeException("Hit end-of-file before finding valid mirror"));
			}else{
				return(MANIFEST_FINISHED);
			}
			
		}else{
		
		//advance the file to the next starting point
			do
			{
				try{
					nextLine = contents.readLine();
					if(nextLine==null)
					{
						isFinished = true;
					}
				}catch(IOException e){
					System.err.println("ManifestMachine get() failed to read advancing line, "+e.getMessage());
				}
			}while(nextLine!=null && !nextLine.equals("--"));
		}
		
		return(line);
	}
}
