package multipart;

import static org.junit.Assert.*;

import org.junit.Test;
import multipart.ManifestMachine;
import java.lang.String;

public class ManifestTest {

	@Test
	public void test_read() {
		ManifestMachine mm = new ManifestMachine("http://mit.edu/6.005/www/fa08/project1/cheerup.jpg.parts");
		try{
			String first = mm.get();
			assertEquals(first, "http://mit.edu/6.005/www/fa08/project1/there/cheerup.jpg");
		}catch(RuntimeException e){
			System.err.println("Failure: "+e.getMessage());
		}
	}	
	
	@Test
	public void test_end_of_manifest() {
		ManifestMachine mm = new ManifestMachine("http://mit.edu/6.005/www/fa08/project1/cheerup.jpg.parts");
		try{
			String first = mm.get();
			String second = mm.get();
			assertEquals(first, "http://mit.edu/6.005/www/fa08/project1/there/cheerup.jpg");
			assertEquals(second, ManifestMachine.MANIFEST_FINISHED);
		}catch(RuntimeException e){
			System.err.println("Failure: "+e.getMessage());
		}
	}	
	
	@Test
	public void test_multipart_easy() {
		ManifestMachine mm = new ManifestMachine("http://mit.edu/6.005/www/fa08/project1/yellowsub.txt-seq.parts");
		try{
			String[] parts = new String[8];
			for(int i=0; i<8; i++)
				parts[i] = mm.get();
			
			assertEquals(parts[0], "http://mit.edu/6.005/www/fa08/project1/here/yellowsub-verse1.txt-seq");
			assertEquals(parts[1], "http://mit.edu/6.005/www/fa08/project1/here/yellowsub-chorus.txt-seq");
			assertEquals(parts[2], "http://mit.edu/6.005/www/fa08/project1/here/yellowsub-verse2.txt-seq");
			assertEquals(parts[3], "http://mit.edu/6.005/www/fa08/project1/here/yellowsub-chorus.txt-seq");
			assertEquals(parts[4], "http://mit.edu/6.005/www/fa08/project1/here/yellowsub-verse3.txt-seq");
			assertEquals(parts[5], "http://mit.edu/6.005/www/fa08/project1/here/yellowsub-chorus.txt-seq");
			assertEquals(parts[6], "http://mit.edu/6.005/www/fa08/project1/here/yellowsub-chorus.txt-seq");
			assertEquals(parts[7], ManifestMachine.MANIFEST_FINISHED);
			
		}catch(RuntimeException e){
			System.err.println("Failure: "+e.getMessage());
		}
	}
	
	@Test
	public void test_multipart_hard() {
		ManifestMachine mm = new ManifestMachine("http://mit.edu/6.005/www/fa08/project1/empty.txt.parts");
		try{
			String[] parts = new String[5];
			for(int i=0; i<5; i++)
			{
				parts[i] = mm.get();
				assertEquals(parts[0], "http://mit.edu/6.005/www/fa08/project1/there/empty.txt");
			}	
			assertEquals(parts[4], ManifestMachine.MANIFEST_FINISHED);
			
		}catch(RuntimeException e){
			System.err.println("Failure: "+e.getMessage());
		}
	}	

}
