package multipart;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Stack;
import java.util.regex.*;
import multipart.DownloadMachine;
import multipart.ManifestMachine;

/**
 * 
 * Parse machine creates a stack of Manifests from which it gets
 * streams that it adds to Download.
 * @author scottyoung
 *
 */
public class ParseMachine extends InputStream {
	
	DownloadMachine dm;
	Stack<ManifestMachine> manifests;
	boolean parseComplete = false;
	/**
	 * Creates a new ParseMachine from a given address
	 */
	public ParseMachine(String url)
	{
		dm = new DownloadMachine();
		manifests = new Stack<ManifestMachine>();
		
		if(url.matches(".*parts"))
		{
			//the file is a manifest, parse it
			manifests.push(new ManifestMachine(url));
			//add a new stream to the DM
			parseComplete = requestStream();
		}else{
			//the file is data
			try{
				dm.add(new URL(url).openStream());
			}catch(IOException e){
				System.err.println("PM could not load: "+url);
			}
		}
	}
	
	/**
	 * Gets a byte from the download machine
	 * @return
	 */
	public int read() throws IOException
	{
		int next = -1;
		if(!parseComplete)
		{
			next = dm.readByte();
			if(next == DownloadMachine.REQUEST_STREAM)
			{
				//if DM tells us it has finished that file, go grab another stream
				//and try again
				parseComplete = requestStream();
				next = read();
			}
		}
		return next;
	}
	
	/**
	 * Gets the next available URL from the top of the stack, if it's data,
	 * stream it to the DM. If it's another manifest, push a new manifest onto
	 * the stack. If it's a MANIFEST_FINISHED flag, then pop that manifest and
	 * continue with the previous manifest. If the manifests stack is empty,
	 * return true (parseComplete = true)
	 */
	private boolean requestStream()
	{
		String nextStream = null;
		while(!manifests.isEmpty() && nextStream == null)
		{
			nextStream = manifests.peek().get();
			if(nextStream.equals(ManifestMachine.MANIFEST_FINISHED))
			{
				//this manifest is complete, pop it
				manifests.pop();
				nextStream = null;
			
			}else if(nextStream.matches(".*parts")){
				//the URL is another manifest, top up the stack
				manifests.push(new ManifestMachine(nextStream));
				nextStream = null;
			
			}else{
				//the URL is a data stream, add it to DM
				try
				{
					dm.add(new URL(nextStream).openStream());
				}catch(IOException e){
					System.err.println("PM failed to load: "+nextStream);
				}
			}
			
			//Do an error check to ensure we never hit an infinitely recursive file
			if(manifests.size() > 1000)
			{
				System.err.println("Overflow error");
				return(true);
			}
		}

		if(manifests.isEmpty())
			return(true);
		else
			return(false);
	}
	
}
