package music;
import music.*;
import java.util.ArrayList;
import java.lang.String;

/**
 * 
 * Music stores a single track
 * 
 * @author scottyoung
 *
 */
public class Music implements MusicElement{
	
	//MUSIC HEADER
	public int trackNo;
	public String title;
	public Key key;
	public double length;
	public double meter;
	public int tempo;
	public String composer;
	
	//MUSIC BODY
	public ArrayList<Voice> voices;
	
	public Music()
	{
		voices = new ArrayList<Voice>();
		trackNo = 0;
		title = "Untitled";
		composer = "Unknown";
		length = .125;
		meter = 1.0;
		tempo = 100;
	}
	
	public boolean verify(double meter)
	{
		for(Voice v : voices)
		{
			if(!v.verify(meter))
				return false;
		}
		return true;
	}	
	
	/**
	 * If the voice exists, adds to it, if not
	 * creates a new voice with the given name
	 * @param name
	 * @return voice (either new or append)
	 */
	public Voice addVoice(String name)
	{
		Voice voice = null;
		for(Voice v: voices)
		{
			if(v.name.equals(name))
			{
				voice = v;
				break;
			}
		}
		
		if(voice == null)
		{
			voice = new Voice(name);
			voices.add(voice);
		}
		
		return(voice);
	}
	
	/**
	 * Prints the music to stderr
	 */
	public void print()
	{
		printHeader();
		for(Voice v : voices)
		{
			v.print();
		}
	}
	
	public void printHeader()
	{
		System.err.println("MUSIC INFO: ");
		System.err.printf("%s: %d\n", "X", trackNo);
		System.err.println("T: "+title);
		System.err.println("C: "+composer);
		System.err.printf("%s: %d\n", "Q", tempo);
		System.err.printf("%s: %f\n", "L", length);
		System.err.printf("%s: %f\n", "M", meter);
		System.err.println("-------------------------");
	}
	
	@Override
	public void accept(MusicVisitor v)
	{
		v.visit(this);
	}
	
}
