package model;

import static org.junit.Assert.*;

import java.util.List;

import org.junit.Test;

import controller.Tuple;

public class _test_Model {

	@Test
	public void test_new_room() {
		Model theModel = new Model();
		theModel.newRoom("room");
		assertNotNull(Room.find("room"));
		assertNull(Room.find("not_here"));
		assertEquals(theModel.newRoom("room"), "room_2");
		assertNotNull(Room.find("room_2"));
	}
	
	@Test
	public void test_new_user() {
		Model theModel = new Model();
		theModel.newUser();
		assertNotNull(User.find("Anonymous"));
		theModel.newUser();
		assertNotNull(User.find("Anonymous_2"));
		assertEquals(theModel.newUser(), "Anonymous_3");
	}
	
	@Test
	public void test_kill_user() {
		Model theModel = new Model();
		theModel.newUser();
		assertTrue(theModel.killUser("Anonymous"));
		assertFalse(theModel.killUser("Anonymous"));
	}
	
	@Test 
	public void test_change_alias() {
		Model theModel = new Model();
		theModel.newUser();
		assertNotNull(User.find("Anonymous"));
		theModel.changeAlias("Anonymous", "newName");
		assertNull(User.find("Anonymous"));
		assertNotNull(User.find("newName"));
		theModel.newUser();
		theModel.changeAlias("Anonymous", "newName");
		assertNotNull(User.find("newName_2"));
	}
	
	@Test
	public void test_join_room() {
		Model theModel = new Model();
		theModel.newUser();
		theModel.newRoom("room");
		assertTrue(theModel.joinRoom("Anonymous", "room"));
		assertFalse(theModel.joinRoom("non-user", "room"));
		assertFalse(theModel.joinRoom("Anonymous", "non-room"));
		theModel.killUser("Anonymous");
		assertFalse(theModel.joinRoom("Anonymous", "room"));
	}
	
	@Test
	public void test_leave_room() {
		Model theModel = new Model();
		theModel.newUser();
		theModel.newRoom("room");
		theModel.joinRoom("Anonymous", "room");
		assertFalse(theModel.leaveRoom("non-user", "room"));
		assertFalse(theModel.leaveRoom("Anonymous", "non-room"));
		assertTrue(theModel.leaveRoom("Anonymous", "room"));
		assertFalse(theModel.leaveRoom("Anonymous", "room"));
		theModel.joinRoom("Anonymous", "room");
		theModel.killUser("Anonymous");
		assertFalse(theModel.leaveRoom("Anonymous", "room"));
	}
	
	@Test
	public void test_write_message(){
		Model theModel = new Model();
		theModel.newUser();
		theModel.newRoom("room");
		assertFalse(theModel.writeMessage("Anonymous", "room", ""));
		theModel.joinRoom("Anonymous", "room");
		assertTrue(theModel.writeMessage("Anonymous", "room", ""));
		assertFalse(theModel.writeMessage("non-user", "room", ""));
		assertFalse(theModel.writeMessage("Anonymous", "non-room", ""));
		theModel.leaveRoom("Anonymous", "room");
		assertFalse(theModel.writeMessage("Anonymous", "room", ""));
		theModel.newUser();
		theModel.newRoom("room2");
		theModel.joinRoom("Anonymous_2", "room2");
		theModel.joinRoom("Anonymous", "room");
		assertFalse(theModel.writeMessage("Anonymous", "room2", ""));
		assertFalse(theModel.writeMessage("Anonymous_2", "room", ""));
	}
	
	@Test
	public void test_get_transcript(){
		Model theModel = new Model();
		theModel.newUser();
		theModel.newRoom("room");
		theModel.joinRoom("Anonymous", "room");
		theModel.writeMessage("Anonymous", "room", "Hello.");
		theModel.writeMessage("Anonymous", "room", "Bye.");
		assertEquals(theModel.getTranscript("room"), "Anonymous: Hello.\nAnonymous: Bye.\n");
	}
	
	@Test
	public void test_get_rooms(){
		Model theModel = new Model();
		for(int i=0; i<100; i++)
			theModel.newRoom("room");
		List<Tuple> theRooms = theModel.getRooms();
		for(Tuple t : theRooms)
		{
			assertEquals(t.name.substring(0, 4), "room");
			assertEquals(t.number, 0);
		}
	}
	
	@Test
	public void test_get_users(){
		Model theModel = new Model();
		for(int i=0; i<100; i++)
			theModel.newUser();
		List<String> theUsers = theModel.getUsers();
		assertTrue(theUsers.contains("Anonymous"));
		for(int i=2; i<101; i++)
		{
			assertTrue(theUsers.contains("Anonymous_"+Integer.toString(i)));
		}
	}	
	
	@Test
	public void test_make_safe(){
		Model theModel = new Model();
		theModel.newRoom("[room]");
		assertNotNull(Room.find("(room)"));
		assertNull(Room.find("[room]"));
		theModel.newUser();
		theModel.changeAlias("Anonymous", "[user][][]");
		assertNull(User.find("[user][][]"));
		assertNotNull(User.find("(user)()()"));
		theModel.joinRoom("(user)()()", "(room)");
		theModel.writeMessage("(user)()()", "(room)", "[transcript[[[[");
		assertEquals(theModel.getTranscript("(room)"), "(user)()(): (transcript((((\n");
	}
	
	@Test
	public void test_get_attendees(){
		Model theModel = new Model();
		theModel.newRoom("room");
		theModel.newRoom("room");
		theModel.newUser();
		theModel.newUser();
		theModel.newUser();
		theModel.newUser();
		theModel.newUser();
		theModel.joinRoom("Anonymous", "room");
		theModel.joinRoom("Anonymous_2", "room");
		theModel.joinRoom("Anonymous_3", "room");
		theModel.joinRoom("Anonymous_4", "room_2");
		theModel.joinRoom("Anonymous_5", "room_2");
		List<String> attendees = theModel.getAttendees("room");
		assertTrue(attendees.size()==3);
	}

}
